@@ -1,8 +1,12 @@
-__version__ = "3.0"
+# pylint: disable=wildcard-import
+__version__ = "3.1"
 
 from .blocking import *
 from .distributions import *
-from .math import logsumexp, logit, invlogit
+from .external import *
+from .glm import *
+from . import gp
+from .math import logaddexp, logsumexp, logit, invlogit, expand_packed_triangular, probit, invprobit
 from .model import *
 from .stats import *
 from .sampling import *
@@ -11,6 +15,7 @@ from .theanof import *
 from .tuning import *
 from .variational import *
 from .vartypes import *
+from .exceptions import *
 from . import sampling
 
 from .debug import *
@@ -22,7 +27,6 @@ from .plots import *
 
 from .tests import test
 
-from . import glm
 from .data import *
 
 import logging
