@@ -5,10 +5,9 @@ Note that gradient based samplers will not work.
 """
 
 
-from pymc3 import *
+import pymc3 as pm
 import theano.tensor as tt
-from theano import as_op
-from numpy import arange, array, empty
+from numpy import arange, array
 
 __all__ = ['disasters_data', 'switchpoint', 'early_mean', 'late_mean', 'rate',
            'disasters']
@@ -23,46 +22,29 @@ disasters_data = array([4, 5, 4, 0, 1, 4, 3, 4, 0, 6, 3, 3, 4, 0, 2, 6,
                         0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 1])
 years = len(disasters_data)
 
-# here is the trick
-
-
-@as_op(itypes=[tt.lscalar, tt.dscalar, tt.dscalar], otypes=[tt.dvector])
-def rateFunc(switchpoint, early_mean, late_mean):
-    ''' Concatenate Poisson means '''
-    out = empty(years)
-    out[:switchpoint] = early_mean
-    out[switchpoint:] = late_mean
-    return out
-
-
-with Model() as model:
+with pm.Model() as model:
 
     # Prior for distribution of switchpoint location
-    switchpoint = DiscreteUniform('switchpoint', lower=0, upper=years)
+    switchpoint = pm.DiscreteUniform('switchpoint', lower=0, upper=years)
     # Priors for pre- and post-switch mean number of disasters
-    early_mean = Exponential('early_mean', lam=1.)
-    late_mean = Exponential('late_mean', lam=1.)
+    early_mean = pm.Exponential('early_mean', lam=1.)
+    late_mean = pm.Exponential('late_mean', lam=1.)
 
     # Allocate appropriate Poisson rates to years before and after current
     # switchpoint location
     idx = arange(years)
-    # theano style:
-    #rate = switch(switchpoint >= idx, early_mean, late_mean)
-    # non-theano style
-    rate = rateFunc(switchpoint, early_mean, late_mean)
-
+    rate = tt.switch(switchpoint >= idx, early_mean, late_mean)
+    
     # Data likelihood
-    disasters = Poisson('disasters', rate, observed=disasters_data)
-
-    # Initial values for stochastic nodes
-    start = {'early_mean': 2., 'late_mean': 3.}
+    disasters = pm.Poisson('disasters', rate, observed=disasters_data)
 
     # Use slice sampler for means
-    step1 = Slice([early_mean, late_mean])
+    step1 = pm.Slice([early_mean, late_mean])
     # Use Metropolis for switchpoint, since it accomodates discrete variables
-    step2 = Metropolis([switchpoint])
-
-    # njobs>1 works only with most recent (mid August 2014) Thenao version:
-    # https://github.com/Theano/Theano/pull/2021
-    tr = sample(1000, tune=500, start=start, step=[step1, step2], njobs=1)
-    traceplot(tr)
+    step2 = pm.Metropolis([switchpoint])
+    
+    # Initial values for stochastic nodes
+    start = {'early_mean': 2., 'late_mean': 3.}
+    
+    tr = pm.sample(1000, tune=500, start=start, step=[step1, step2], njobs=2)
+    pm.traceplot(tr)
