@@ -5,16 +5,18 @@ import pandas as pd
 import itertools
 import sys
 import warnings
+from collections import namedtuple
 from .model import modelcontext
+from .util import get_default_varnames
+from pymc3.theanof import floatX
 
 from scipy.misc import logsumexp
 from scipy.stats.distributions import pareto
 
-import pymc3 as pm
 from .backends import tracetab as ttab
 
 __all__ = ['autocorr', 'autocov', 'dic', 'bpic', 'waic', 'loo', 'hpd', 'quantiles',
-           'mc_error', 'summary', 'df_summary']
+           'mc_error', 'summary', 'df_summary', 'compare']
 
 
 def statfunc(f):
@@ -58,18 +60,32 @@ def statfunc(f):
 @statfunc
 def autocorr(x, lag=1):
     """Sample autocorrelation at specified lag.
-    The autocorrelation is the correlation of x_i with x_{i+lag}.
-    """
 
+    Parameters
+    ----------
+    x : Numpy array
+        An array containing MCMC samples
+    lag : int
+        The desidered lag to take in consideration
+    """
     S = autocov(x, lag)
     return S[0, 1] / np.sqrt(np.prod(np.diag(S)))
 
 
 @statfunc
 def autocov(x, lag=1):
-    """
-    Sample autocovariance at specified lag.
-    The autocovariance is a 2x2 matrix with the variances of
+    """Sample autocovariance at specified lag.
+
+    Parameters
+    ----------
+    x : Numpy array
+        An array containing MCMC samples
+    lag : int
+        The desidered lag to take in consideration
+
+    Returns
+    -------
+    2x2 matrix with the variances of
     x[:-lag] and x[lag:] in the diagonal and the autocovariance
     on the off-diagonal.
     """
@@ -83,9 +99,19 @@ def autocov(x, lag=1):
 
 
 def dic(trace, model=None):
-    """
-    Calculate the deviance information criterion of the samples in trace from model
-    Read more theory here - in a paper by some of the leading authorities on Model Selection - dx.doi.org/10.1111/1467-9868.00353
+    """Calculate the deviance information criterion of the samples in trace from model
+    Read more theory here - in a paper by some of the leading authorities on Model Selection -
+    dx.doi.org/10.1111/1467-9868.00353
+
+    Parameters
+    ----------
+    trace : result of MCMC run
+    model : PyMC Model
+        Optional model. Default None, taken from context.
+
+    Returns
+    -------
+    `float` representing the deviance information criterion of the model and trace
     """
     model = modelcontext(model)
 
@@ -98,42 +124,67 @@ def dic(trace, model=None):
     return 2 * mean_deviance - deviance_at_mean
 
 
-def log_post_trace(trace, model):
-    '''
-    Calculate the elementwise log-posterior for the sampled trace.
-    '''
-    return np.vstack([obs.logp_elemwise(pt) for obs in model.observed_RVs] for pt in trace)
-
-
-def waic(trace, model=None, n_eff=False):
-    """
-    Calculate the widely available information criterion, its standard error
-    and the effective number of parameters of the samples in trace from model.
-    Read more theory here - in a paper by some of the leading authorities on
-    Model Selection - dx.doi.org/10.1111/1467-9868.00353
-
+def _log_post_trace(trace, model):
+    """Calculate the elementwise log-posterior for the sampled trace.
 
     Parameters
     ----------
     trace : result of MCMC run
     model : PyMC Model
         Optional model. Default None, taken from context.
-    n_eff: bool
-        if True the effective number parameters will be returned.
-        Default False
-
 
     Returns
     -------
+    logp : array of shape (n_samples, n_observations)
+        The contribution of the observations to the logp of the whole model.
+    """
+    def logp_vals_point(pt):
+        if len(model.observed_RVs) == 0:
+            return floatX(np.array([], dtype='d'))
+
+        logp_vals = []
+        for var in model.observed_RVs:
+            logp = var.logp_elemwise(pt)
+            if var.missing_values:
+                logp = logp[~var.observations.mask]
+            logp_vals.append(logp.ravel())
+
+        return np.concatenate(logp_vals)
+
+    logp = (logp_vals_point(pt) for pt in trace)
+    return np.stack(logp)
+
+
+def waic(trace, model=None, pointwise=False):
+    """Calculate the widely available information criterion, its standard error
+    and the effective number of parameters of the samples in trace from model.
+    Read more theory here - in a paper by some of the leading authorities on
+    Model Selection - dx.doi.org/10.1111/1467-9868.00353
+
+    Parameters
+    ----------
+    trace : result of MCMC run
+    model : PyMC Model
+        Optional model. Default None, taken from context.
+    pointwise: bool
+        if True the pointwise predictive accuracy will be returned.
+        Default False
+
+    Returns
+    -------
+    namedtuple with the following elements:
     waic: widely available information criterion
     waic_se: standard error of waic
-    p_waic: effective number parameters, only if n_eff True
+    p_waic: effective number parameters
+    waic_i: and array of the pointwise predictive accuracy, only if pointwise True
     """
     model = modelcontext(model)
 
-    log_py = log_post_trace(trace, model)
+    log_py = _log_post_trace(trace, model)
+    if log_py.size == 0:
+        raise ValueError('The model does not contain observed values.')
 
-    lppd_i = logsumexp(log_py, axis = 0, b = 1.0 / log_py.shape[0])
+    lppd_i = logsumexp(log_py, axis=0, b=1.0 / log_py.shape[0])
 
     vars_lpd = np.var(log_py, axis=0)
     if np.any(vars_lpd > 0.4):
@@ -149,39 +200,41 @@ def waic(trace, model=None, n_eff=False):
 
     p_waic = np.sum(vars_lpd)
 
-
-    if n_eff:
-        return waic, waic_se, p_waic
+    if pointwise:
+        WAIC_r = namedtuple('WAIC_r', 'WAIC, WAIC_se, p_WAIC, WAIC_i')
+        return WAIC_r(waic, waic_se, p_waic, waic_i)
     else:
-        return waic, waic_se
+        WAIC_r = namedtuple('WAIC_r', 'WAIC, WAIC_se, p_WAIC')
+        return WAIC_r(waic, waic_se, p_waic)
 
 
-def loo(trace, model=None, n_eff=False):
-    """
-    Calculates leave-one-out (LOO) cross-validation for out of sample predictive
+def loo(trace, model=None, pointwise=False):
+    """Calculates leave-one-out (LOO) cross-validation for out of sample predictive
     model fit, following Vehtari et al. (2015). Cross-validation is computed using
     Pareto-smoothed importance sampling (PSIS).
 
-
     Parameters
     ----------
     trace : result of MCMC run
     model : PyMC Model
         Optional model. Default None, taken from context.
-    n_eff: bool
-        if True the effective number parameters will be computed and returned.
+    pointwise: bool
+        if True the pointwise predictive accuracy will be returned.
         Default False
 
-
     Returns
     -------
-    elpd_loo: log pointwise predictive density calculated via approximated LOO cross-validation
-    elpd_loo_se: standard error of waic elpd_loo
-    p_loo: effective number parameters, only if n_eff True
+    namedtuple with the following elements:
+    loo: approximated Leave-one-out cross-validation
+    loo_se: standard error of loo
+    p_loo: effective number of parameters
+    loo_i: and array of the pointwise predictive accuracy, only if pointwise True
     """
     model = modelcontext(model)
 
-    log_py = log_post_trace(trace, model)
+    log_py = _log_post_trace(trace, model)
+    if log_py.size == 0:
+        raise ValueError('The model does not contain observed values.')
 
     # Importance ratios
     r = np.exp(-log_py)
@@ -217,29 +270,40 @@ def loo(trace, model=None, n_eff=False):
     r_sorted[q80:] = np.vstack(expvals).T
     # Unsort ratios (within columns) before using them as weights
     r_new = np.array([r[np.argsort(i)]
-                      for r, i in zip(r_sorted, np.argsort(r, axis=0))])
+                      for r, i in zip(r_sorted.T, np.argsort(r.T, axis=1))]).T
 
     # Truncate weights to guarantee finite variance
     w = np.minimum(r_new, r_new.mean(axis=0) * S**0.75)
 
-    loo_lppd_i = -2.0 * logsumexp(log_py, axis = 0, b = w / np.sum(w, axis = 0))
+    loo_lppd_i = - 2. * logsumexp(log_py, axis=0, b=w / np.sum(w, axis=0))
 
     loo_lppd_se = np.sqrt(len(loo_lppd_i) * np.var(loo_lppd_i))
 
     loo_lppd = np.sum(loo_lppd_i)
 
+    lppd = np.sum(logsumexp(log_py, axis=0, b=1. / log_py.shape[0]))
 
-    if n_eff:
-        p_loo = np.sum(np.log(np.mean(py, axis=0))) - loo_lppd
-        return loo_lppd, loo_lppd_se, p_loo
+    p_loo = lppd + (0.5 * loo_lppd)
+
+    if pointwise:
+        LOO_r = namedtuple('LOO_r', 'LOO, LOO_se, p_LOO, LOO_i')
+        return LOO_r(loo_lppd, loo_lppd_se, p_loo, loo_lppd_i)
     else:
-        return loo_lppd, loo_lppd_se
+        LOO_r = namedtuple('LOO_r', 'LOO, LOO_se, p_LOO')
+        return LOO_r(loo_lppd, loo_lppd_se, p_loo)
 
 
 def bpic(trace, model=None):
     """
     Calculates Bayesian predictive information criterion n of the samples in trace from model
-    Read more theory here - in a paper by some of the leading authorities on Model Selection - dx.doi.org/10.1111/1467-9868.00353
+    Read more theory here - in a paper by some of the leading authorities on Model Selection -
+    dx.doi.org/10.1111/1467-9868.00353
+
+    Parameters
+    ----------
+    trace : result of MCMC run
+    model : PyMC Model
+        Optional model. Default None, taken from context.
     """
     model = modelcontext(model)
 
@@ -252,28 +316,102 @@ def bpic(trace, model=None):
     return 3 * mean_deviance - 2 * deviance_at_mean
 
 
+def compare(traces, models, ic='WAIC'):
+    """Compare models based on the widely available information criterion (WAIC)
+    or leave-one-out (LOO) cross-validation.
+    Read more theory here - in a paper by some of the leading authorities on
+    Model Selection - dx.doi.org/10.1111/1467-9868.00353
+
+    Parameters
+    ----------
+    traces : list of PyMC3 traces
+    models : list of PyMC3 models
+        in the same order as traces.
+    ic : string
+        Information Criterion (WAIC or LOO) used to compare models.
+        Default WAIC.
+
+    Returns
+    -------
+    A DataFrame, ordered from lowest to highest IC. The index reflects
+    the order in which the models are passed to this function. The columns are:
+    IC : Information Criteria (WAIC or LOO).
+        Smaller IC indicates higher out-of-sample predictive fit ("better" model).
+        Default WAIC.
+    pIC : Estimated effective number of parameters.
+    dIC : Relative difference between each IC (WAIC or LOO)
+    and the lowest IC (WAIC or LOO).
+        It's always 0 for the top-ranked model.
+    weight: Akaike weights for each model.
+        This can be loosely interpreted as the probability of each model
+        (among the compared model) given the data. Be careful that these
+        weights are based on point estimates of the IC (uncertainty is ignored).
+    SE : Standard error of the IC estimate.
+        For a "large enough" sample size this is an estimate of the uncertainty
+        in the computation of the IC.
+    dSE : Standard error of the difference in IC between each model and
+    the top-ranked model.
+        It's always 0 for the top-ranked model.
+    warning : A value of 1 indicates that the computation of the IC may not be
+    reliable see http://arxiv.org/abs/1507.04544 for details.
+    """
+    if ic == 'WAIC':
+        ic_func = waic
+        df_comp = pd.DataFrame(index=np.arange(len(models)),
+                               columns=['WAIC', 'pWAIC', 'dWAIC', 'weight',
+                               'SE', 'dSE', 'warning'])
+    elif ic == 'LOO':
+        ic_func = loo
+        df_comp = pd.DataFrame(index=np.arange(len(models)),
+                               columns=['LOO', 'pLOO', 'dLOO', 'weight',
+                               'SE', 'dSE', 'warning'])
+    else:
+        raise NotImplementedError(
+            'The information criterion {} is not supported.'.format(ic))
+
+    warns = np.zeros(len(models))
+
+    c = 0
+
+    def add_warns(*args):
+        warns[c] = 1
+
+    with warnings.catch_warnings():
+        warnings.showwarning = add_warns
+        warnings.filterwarnings('always')
+
+        ics = []
+        for c, (t, m) in enumerate(zip(traces, models)):
+            ics.append((c, ic_func(t, m, pointwise=True)))
+
+    ics.sort(key=lambda x: x[1][0])
+
+    min_ic = ics[0][1][0]
+    Z = np.sum([np.exp(-0.5 * (x[1][0] - min_ic)) for x in ics])
+
+    for idx, res in ics:
+        diff = ics[0][1][3] - res[3]
+        d_ic = np.sum(diff)
+        d_se = len(diff) ** 0.5 * np.var(diff)
+        weight = np.exp(-0.5 * (res[0] - min_ic)) / Z
+        df_comp.at[idx] = (res[0], res[2], abs(d_ic), weight, res[1],
+                           d_se, warns[idx])
+
+    return df_comp.sort_values(by=ic)
+
+
 def make_indices(dimensions):
     # Generates complete set of indices for given dimensions
-
     level = len(dimensions)
-
     if level == 1:
         return list(range(dimensions[0]))
-
     indices = [[]]
-
     while level:
-
         _indices = []
-
         for j in range(dimensions[level - 1]):
-
             _indices += [[j] + i for i in indices]
-
         indices = _indices
-
         level -= 1
-
     try:
         return [tuple(i) for i in indices]
     except TypeError:
@@ -286,7 +424,6 @@ def calc_min_interval(x, alpha):
 
     Assumes that x is sorted numpy array.
     """
-
     n = len(x)
     cred_mass = 1.0 - alpha
 
@@ -317,7 +454,6 @@ def hpd(x, alpha=0.05, transform=lambda x: x):
           Function to transform data (defaults to identity)
 
     """
-
     # Make a copy of trace
     x = transform(x.copy())
 
@@ -356,18 +492,21 @@ def hpd(x, alpha=0.05, transform=lambda x: x):
 
 @statfunc
 def mc_error(x, batches=5):
-    """
-    Calculates the simulation standard error, accounting for non-independent
-    samples. The trace is divided into batches, and the standard deviation of
-    the batch means is calculated.
+    R"""Calculates the simulation standard error, accounting for non-independent
+        samples. The trace is divided into batches, and the standard deviation of
+        the batch means is calculated.
 
-    :Arguments:
-      x : Numpy array
-          An array containing MCMC samples
-      batches : integer
-          Number of batches
-    """
+    Parameters
+    ----------
+    x : Numpy array
+              An array containing MCMC samples
+    batches : integer
+              Number of batches
 
+    Returns
+    -------
+    `float` representing the error
+    """
     if x.ndim > 1:
 
         dims = np.shape(x)
@@ -395,17 +534,21 @@ def mc_error(x, batches=5):
 
 @statfunc
 def quantiles(x, qlist=(2.5, 25, 50, 75, 97.5), transform=lambda x: x):
-    """Returns a dictionary of requested quantiles from array
+    R"""Returns a dictionary of requested quantiles from array
 
-    :Arguments:
-      x : Numpy array
-          An array containing MCMC samples
-      qlist : tuple or list
-          A list of desired quantiles (defaults to (2.5, 25, 50, 75, 97.5))
-      transform : callable
-          Function to transform data (defaults to identity)
+    Parameters
+    ----------
+    x : Numpy array
+        An array containing MCMC samples
+    qlist : tuple or list
+        A list of desired quantiles (defaults to (2.5, 25, 50, 75, 97.5))
+    transform : callable
+        Function to transform data (defaults to identity)
+
+    Returns
+    -------
+    `dictionary` with the quantiles {quantile: value}
     """
-
     # Make a copy of trace
     x = transform(x.copy())
 
@@ -427,8 +570,9 @@ def quantiles(x, qlist=(2.5, 25, 50, 75, 97.5), transform=lambda x: x):
         _log.warning("Too few elements for quantile calculation")
 
 
-def df_summary(trace, varnames=None, stat_funcs=None, extend=False, include_transformed=False,
-               alpha=0.05, batches=None):
+def df_summary(trace, varnames=None, transform=lambda x: x, stat_funcs=None, 
+               extend=False, include_transformed=False,
+               alpha=0.05, start=0, batches=None):
     R"""Create a data frame with summary statistics.
 
     Parameters
@@ -436,6 +580,8 @@ def df_summary(trace, varnames=None, stat_funcs=None, extend=False, include_tran
     trace : MultiTrace instance
     varnames : list
         Names of variables to include in summary
+    transform : callable
+        Function to transform data (defaults to identity)
     stat_funcs : None or list
         A list of functions used to calculate statistics. By default,
         the mean, standard deviation, simulation standard error, and
@@ -446,6 +592,7 @@ def df_summary(trace, varnames=None, stat_funcs=None, extend=False, include_tran
         corresponds to sampling iterations and the second axis
         represents the flattened variable (e.g., x__0, x__1,...). Each
         function should return either
+
         1) A `pandas.Series` instance containing the result of
            calculating the statistic along the first axis. The name
            attribute will be taken as the name of the statistic.
@@ -463,51 +610,52 @@ def df_summary(trace, varnames=None, stat_funcs=None, extend=False, include_tran
     alpha : float
         The alpha level for generating posterior intervals. Defaults
         to 0.05. This is only meaningful when `stat_funcs` is None.
+    start : int
+        The starting index from which to summarize (each) chain. Defaults
+        to zero.
     batches : None or int
         Batch size for calculating standard deviation for non-independent
         samples. Defaults to the smaller of 100 or the number of samples.
         This is only meaningful when `stat_funcs` is None.
 
-
     See also
     --------
     summary : Generate a pretty-printed summary of a trace.
 
-
     Returns
     -------
     `pandas.DataFrame` with summary statistics for each variable
 
-
     Examples
     --------
-    >>> import pymc3 as pm
-    >>> trace.mu.shape
-    (1000, 2)
-    >>> pm.df_summary(trace, ['mu'])
-               mean        sd  mc_error     hpd_5    hpd_95
-    mu__0  0.106897  0.066473  0.001818 -0.020612  0.231626
-    mu__1 -0.046597  0.067513  0.002048 -0.174753  0.081924
+    .. code:: ipython
+
+        >>> import pymc3 as pm
+        >>> trace.mu.shape
+        (1000, 2)
+        >>> pm.df_summary(trace, ['mu'])
+                   mean        sd  mc_error     hpd_5    hpd_95
+        mu__0  0.106897  0.066473  0.001818 -0.020612  0.231626
+        mu__1 -0.046597  0.067513  0.002048 -0.174753  0.081924
 
     Other statistics can be calculated by passing a list of functions.
 
-    >>> import pandas as pd
-    >>> def trace_sd(x):
-    ...     return pd.Series(np.std(x, 0), name='sd')
-    ...
-    >>> def trace_quantiles(x):
-    ...     return pd.DataFrame(pm.quantiles(x, [5, 50, 95]))
-    ...
-    >>> pm.df_summary(trace, ['mu'], stat_funcs=[trace_sd, trace_quantiles])
-                 sd         5        50        95
-    mu__0  0.066473  0.000312  0.105039  0.214242
-    mu__1  0.067513 -0.159097 -0.045637  0.062912
+    .. code:: ipython
+
+        >>> import pandas as pd
+        >>> def trace_sd(x):
+        ...     return pd.Series(np.std(x, 0), name='sd')
+        ...
+        >>> def trace_quantiles(x):
+        ...     return pd.DataFrame(pm.quantiles(x, [5, 50, 95]))
+        ...
+        >>> pm.df_summary(trace, ['mu'], stat_funcs=[trace_sd, trace_quantiles])
+                     sd         5        50        95
+        mu__0  0.066473  0.000312  0.105039  0.214242
+        mu__1  0.067513 -0.159097 -0.045637  0.062912
     """
     if varnames is None:
-        if include_transformed:
-            varnames = [name for name in trace.varnames]
-        else:
-            varnames = [name for name in trace.varnames if not name.endswith('_')]
+        varnames = get_default_varnames(trace.varnames, include_transformed=include_transformed)
 
     if batches is None:
         batches = min([100, len(trace)])
@@ -524,7 +672,7 @@ def df_summary(trace, varnames=None, stat_funcs=None, extend=False, include_tran
 
     var_dfs = []
     for var in varnames:
-        vals = trace.get_values(var, combine=True)
+        vals = transform(trace.get_values(var, burn=start, combine=True))
         flat_vals = vals.reshape(vals.shape[0], -1)
         var_df = pd.concat([f(flat_vals) for f in stat_funcs], axis=1)
         var_df.index = ttab.create_flat_names(var, vals.shape[1:])
@@ -538,8 +686,8 @@ def _hpd_df(x, alpha):
     return pd.DataFrame(hpd(x, alpha), columns=cnames)
 
 
-def summary(trace, varnames=None, alpha=0.05, start=0, batches=None, roundto=3,
-            include_transformed=False, to_file=None):
+def summary(trace, varnames=None, transform=lambda x: x, alpha=0.05, start=0,
+            batches=None, roundto=3, include_transformed=False, to_file=None):
     R"""
     Generate a pretty-printed summary of the node.
 
@@ -550,6 +698,8 @@ def summary(trace, varnames=None, alpha=0.05, start=0, batches=None, roundto=3,
     varnames : list of strings
       List of variables to summarize. Defaults to None, which results
       in all variables summarized.
+    transform : callable
+      Function to transform data (defaults to identity)
     alpha : float
       The alpha level for generating posterior intervals. Defaults to
       0.05.
@@ -565,15 +715,11 @@ def summary(trace, varnames=None, alpha=0.05, start=0, batches=None, roundto=3,
     include_transformed : bool
       Flag for summarizing automatically transformed variables in addition to
       original variables (defaults to False).
-    tofile : None or string
+    to_file : None or string
       File to write results to. If not given, print to stdout.
-
     """
     if varnames is None:
-        if include_transformed:
-            varnames = [name for name in trace.varnames]
-        else:
-            varnames = [name for name in trace.varnames if not name.endswith('_')]
+        varnames = get_default_varnames(trace.varnames, include_transformed=include_transformed)
 
     if batches is None:
         batches = min([100, len(trace)])
@@ -588,7 +734,7 @@ def summary(trace, varnames=None, alpha=0.05, start=0, batches=None, roundto=3,
 
     for var in varnames:
         # Extract sampled values
-        sample = trace.get_values(var, burn=start, combine=True)
+        sample = transform(trace.get_values(var, burn=start, combine=True))
 
         fh.write('\n%s:\n\n' % var)
 
